#ifndef LAUNCH_H_INCLUDED
#define LAUNCH_H_INCLUDED

#include <string>
#include <vector>
#include <dll_loader.h>
#include <build_info.h>
#include <assert_static.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>


bool runningWOW64() //test if process is running under WOW64 (reference http://msdn.microsoft.com/en-us/library/ms684139(VS.85).aspx)
{
    //dynamically load windows API function
    typedef BOOL (WINAPI *IsWow64ProcessFun)(HANDLE hProcess, PBOOL Wow64Process);

    static const IsWow64ProcessFun isWow64Process = util::getDllFun<IsWow64ProcessFun>(L"kernel32.dll", "IsWow64Process");

    if (isWow64Process)
    {
        BOOL isWow64 = FALSE;
        if (isWow64Process(::GetCurrentProcess(), &isWow64))
            return isWow64 == TRUE;
    }

    return false;
}


int launch(const std::wstring& exe32bit, const std::wstring& exe64bit) //returns called process' return code
{
    //get unicode commandline
    std::vector<std::wstring> argv;
    {
        LPTSTR cmdLine = ::GetCommandLine();

        //parse commandline
        int argc = 0;
        LPWSTR* tmp = ::CommandLineToArgvW(cmdLine, &argc);
        for (int i = 0; i < argc; ++i)
            argv.push_back(tmp[i]);
        ::LocalFree(tmp);
    }

    //redirect commandline
    std::wstring redCmdLine;
    if (!argv.empty())
    {
        //replace executable name
        size_t pos = argv[0].rfind(L'\\');
        if (pos != std::wstring::npos)
            redCmdLine = argv[0].substr(0, pos + 1);

        assert_static(util::is32BitBuild);
        redCmdLine += runningWOW64() ? exe64bit : exe32bit;

        redCmdLine = L"\"" + redCmdLine+ L"\"";

        for (std::vector<std::wstring>::const_iterator i = argv.begin() + 1; i != argv.end(); ++i)
            redCmdLine += L" \"" + *i + L"\"";
    }


    //::MessageBox(0, redCmdLine.c_str(), L"Info", 0);

    STARTUPINFO startupInfo = {};
    startupInfo.cb = sizeof(startupInfo);

    PROCESS_INFORMATION procInfo = {};
    BOOL rv = ::CreateProcess(NULL,           //__in_opt     LPCTSTR lpApplicationName,
                              &redCmdLine[0], //__inout_opt  LPTSTR lpCommandLine,
                              NULL,           //__in_opt     LPSECURITY_ATTRIBUTES lpProcessAttributes,
                              NULL,           //__in_opt     LPSECURITY_ATTRIBUTES lpThreadAttributes,
                              TRUE,           //__in         BOOL bInheritHandles,
                              0,              //__in         DWORD dwCreationFlags,
                              NULL,           //__in_opt     LPVOID lpEnvironment,
                              NULL,           //__in_opt     LPCTSTR lpCurrentDirectory,
                              &startupInfo,   //__in         LPSTARTUPINFO lpStartupInfo,
                              &procInfo);     //__out        LPPROCESS_INFORMATION lpProcessInformation
    if (rv == 0)
    {
        ::MessageBox(0, (L"Error launching: " + redCmdLine).c_str(), L"Error", 0);
        return -1;
    }

    //synchronous call
    ::WaitForSingleObject(procInfo.hProcess, INFINITE);

    //redirect return code
    DWORD exitCode = 0;
    if (!::GetExitCodeProcess(procInfo.hProcess, // __in   HANDLE hProcess,
                              &exitCode) ||      //__out  LPDWORD lpExitCode
        exitCode == STILL_ACTIVE)
        exitCode = -100;

    ::CloseHandle(procInfo.hProcess);
    ::CloseHandle(procInfo.hThread);

    return exitCode;
}

#endif // LAUNCH_H_INCLUDED
